/*
 * (C) Copyright Pattern Recognition Group Delft 1990. All rights reserved.
 *
 * PATTERN.H : General include file with all data structure definitions.
 *
 * Made by : W. Schmidt
 *           M. Kraaijveld.
 *
 * Revision: 08/08/96 Dick de Ridder
 *           Added a function to patutils.c. #ifdef DR1.
 *
 * SccsId  : %Z%$RCSfile: pattern.h,v $ V%Q% $Revision: 1.13 $ $Date: 1996/10/07 06:56:38 $
 */

#define DR1

/*
 * Set flags.
 */

#define IOSET            (long) 0x0001  
#define LABELSET         (long) 0x0000
#define LEARNSET         (long) 0x0002
#define TESTSET          (long) 0x0000
#define STATIST          (long) 0x0004
#define NOSTATIST        (long) 0x0000

/*
 * Sample flags.
 */

#define SAMPLE_ENABLED   (long) 0x0001
#define SAMPLE_DISABLED  (long) 0x0000

/*
 * System data R/W flags.
 */

#define LEARNREAD       (long) 0x0001
#define LEARNWRITE      (long) 0x0000
#define SAMPLEREAD      (long) 0x0002
#define SAMPLEWRITE     (long) 0x0000
#define ULEARNREAD      (long) 0x0004
#define ULEARNWRITE     (long) 0x0000
#define USAMPLEREAD     (long) 0x0008
#define USAMPLEWRITE    (long) 0x0000
#define PERF_INIT_DONE  (long) 0x0010

/* 
 * Data structure holding distribution information.
 */

typedef struct distribution
{
   long Type;
   double Mean;
   double Min;
   double Max;
   double Variance;
} DISTRIBUTION;

/*
 * Data structure with all statistics about data set.
 */

typedef struct statistics
{
   double *Apriori;
   double **Costs;
   struct distribution **FeaturesClass;
   struct distribution *Class;
   struct distribution *Feature;
} STATISTICS;

/*
 * structure holding information of specific data point.
 */

typedef struct sample
{
   long SampleId;                /* Sample identification. */
   long SampleFlag;              /* Sample flag. */
   double *Input;                /* Input vector. */
   union out                     /* Output vector or label. */
   {
      double *Vector;
      int Label;
   } Output;
   union system_data SystemData; /* Union of data used by library. */
   union system_data UserData;   /* Union of data for users. */   
   struct sample *Next;          /* Pointer to next data sample. */
} SAMPLE;
   
/*
 * Structure holding general data set information.
 */

typedef struct dataset
{
   char SetText[256];            /* User description of learning set. */
   char CreationDate[32];        /* Dataset set creation date. */
   long SetId;                   /* Dataset set identification number. */
   long SetFlag;                 /* Dataset set flag (label or i/o patterns.*/
   long NumSamples;              /* Total number of samples in file. */
   long NumInputs;               /* Total number of input features. */
   long NumOutputs;              /* Total number of outputs (or label). */
   char **InputDescr;            /* Verbal description of input features. */
   char **OutputDescr;           /* Verbal description of outputs (or label).*/ 
   long SystemDataRWFlag;        /* Is system data already in use. */
   union system_data SystemData; /* Union of data used by library. */
   union system_data UserData;   /* Union of data for users. */
   struct sample *FirstSample;   /* Pointer to first sample. */
   struct statistics Stat;       /* Structure with statistics. */
} DATASET;

/* sys/patalloc.c */
DATASET  *malloc_dataset 			(void);
void 			free_dataset 				(DATASET *Ptr);
SAMPLE   *malloc_sample 			(void);
void 			free_sample 				(SAMPLE *Ptr);
void 			free_system_dataset (void);

/* sys/patio.c */
int 			fprintf_dataset 		(FILE *stream, DATASET *lset);
int 			fprintf_compressed_dataset 
															(char *FileName, DATASET *lset);
DATASET  *fscanf_dataset 			(FILE *stream);
DATASET  *fscanf_compressed_dataset 
															(char *FileName);
DATASET  *load_dataset 				(char *FileName);

/* sys/patutils.c */
void 			delete_dataset 			(DATASET *lset);
DATASET  *labelset_to_ioset 	(DATASET *label_set);
DATASET  *ioset_to_labelset 	(DATASET *iset);

int 			scale_output_dataset 			
															(DATASET *lset, double NewMin, double NewMax, int Index, double *OutScale, double *OutOffset);
int 			transform_output_dataset 	
															(DATASET *lset, int Index, double Scale, double Offset);

int 			random_order_dataset 		
															(DATASET *dset);

DATASET  *subspace_dataset 		(DATASET *iset, double **v, int subdim);

#ifdef DR1
int 			normalize_one_input_dataset 
															(DATASET *lset, int index, double scale, double offset);
int 			normalize_one_output_dataset 
															(DATASET *lset, int index, double scale, double offset);

#define 	normalize_input_dataset(s)  		{ int i; for (i=1; i<=s->NumInputs; i++) normalize_one_input_dataset(s,i,1.0,0.0);   }
#define		normalize_output_dataset(s)			{ int	i; for (i=1; i<=s->NumOutputs; i++) normalize_one_output_dataset(s,i,1.0,0.0); }
#define		normalize_dataset(s)						{ normalize_input_dataset(s); normalize_output_dataset(s);                           }
#endif

#undef DR1

/*
 * End of include file.
 */

